#ifndef __rp3d_classes__
#define __rp3d_classes__

namespace rp3d

#define defClass(_class_,_base_) type _class_ extends _base_ : end type


' This abstract class represent a body of the physics engine.
' You should not instantiante this class but instantiate
' the CollisionBody or RigidBody classes instead.
defClass(Body,Object)

 ' This class represents a body that is able to collide with others bodies.
 ' This class inherits from the Body class.
 defClass(CollisionBody,Body)

  ' This class represents a rigid body of the physics engine.
  ' A rigid body is a non-deformable body that has a constant mass.
  ' This class inherits from the CollisionBody class.
  defClass(RigidBody,CollisionBody)

' This abstract class represents the collision shape associated with a
' body that is used during the narrow-phase collision detection.
defClass(CollisionShape,Object)

 ' This abstract class represents a convex collision shape associated with a
 ' body that is used during the narrow-phase collision detection.
 defClass(ConvexShape,CollisionShape)

  ' This class represents a sphere collision shape 
  ' that is centered at the origin and defined by its radius. 
  ' This collision shape does not have an explicit object margin distance.
  ' The margin is implicitly the radius of the sphere.
  ' Therefore, no need to specify an object margin for a sphere shape.
  defClass(SphereShape,ConvexShape)

  ' This class represents a capsule collision shape that is defined around the Y axis.
  ' A capsule shape can be seen as the convex hull of two spheres.
  ' The capsule shape is defined by its radius (radius of the two spheres of the capsule)
  ' and its height (distance between the centers of the two spheres).
  ' This collision shape does not have an explicit object margin distance.
  ' The margin is implicitly the radius and height of the shape.
  ' Therefore, no need to specify an object margin for a capsule shape.
  defClass(CapsuleShape,ConvexShape)

  ' This abstract class represents a convex polyhedron collision shape 
  ' associated with a body that is used during the narrow-phase collision detection.
  defClass(ConvexPolyhedronShape,ConvexShape)

   ' This class represents a convex mesh shape.
   ' In order to create a convex mesh shape, you need to indicate the local-space position of the mesh vertices.
   ' The center of mass of the shape will be at the origin of the local-space geometry that you use to create the mesh.
   defClass(ConvexMeshShape,ConvexPolyhedronShape)

   ' This class represents a triangle collision shape that is centered at the origin and defined three points.
   ' A user cannot instanciate an object of this class.
   ' This class is for internal use only.
   ' Instances of this class are created when the user creates an HeightFieldShape and a ConcaveMeshShape
   defClass(TriangleShape,ConvexPolyhedronShape)

   ' This class represents a 3D box shape. Those axis are unit length.
   ' The three extents are half-widths of the box along the three axis x, y, z local axis.
   ' The "transform" of the corresponding body will give an orientation and a position to the box.
   defClass(BoxShape,ConvexPolyhedronShape)

 ' This abstract class represents a concave collision shape associated with a
 ' body that is used during the narrow-phase collision detection.
 defClass(ConcaveShape,CollisionShape)

  ' This class represents a static concave mesh shape.
  ' Note that collision detection with a concave mesh shape can be very expensive.
  ' You should only use this shape for a static mesh.
  defClass(ConcaveMeshShape,ConcaveShape)

  ' This class represents a static height field that can be used to represent a terrain.
  ' The height field is made of a grid with rows and columns with a height value at each grid point.
  defClass(HeightFieldShape,ConcaveShape)

' This class represent a world where it is possible to move bodies by hand 
' and to test collision between each other.
' In this kind of world, the bodies movement is not computed using the laws of physics.
defclass(CollisionWorld,Object)

 ' This class represents a dynamics world. This class inherits from the CollisionWorld class.
 ' In a dynamics world, bodies can collide and their movements are simulated using the laws of physics.
 defclass(DynamicsWorld,CollisionWorld)


' Class ProxyShape
' The CollisionShape instances are supposed to be unique for memory optimization.
' For instance, consider two rigid bodies with the same sphere collision shape.
' In this situation, we will have a unique instance of SphereShape but we need 
' to differentiate between the two instances during the collision detection.
' They do not have the same position in the world and they do not belong to the same rigid body.
' The ProxyShape class is used for that purpose by attaching a rigid body with one of its collision shape.
' A body can have multiple proxy shapes (one for each collision shape attached to the body).
type ProxyShape as any

' This abstract class represents a joint between two bodies.
defclass(Joint,Object)

 ' This class represents a fixed joint that is used to forbid
 ' any translation or rotation between two bodies.
 defclass(FixedJoint,Joint)

 ' This class represents a hinge joint that allows 
 ' arbitrary rotation between two bodies around a single axis.
 ' This joint has one degree of freedom.
 ' It can be useful to simulate doors or pendulumns.
 defclass(HingeJoint,Joint)

 ' This class represents a slider joint.
 ' This joint has a one degree of freedom.
 ' It only allows relative translation of the bodies
 ' along a single direction and no rotation.
 defclass(SliderJoint,Joint)

 ' This class represents a ball-and-socket joint 
 ' that allows arbitrary rotation between two bodies.
 ' This joint has three degrees of freedom.
 ' It can be used to create a chain of bodies for instance.
 defclass(BallAndSocketJoint,Joint)


#undef defClass

end namespace

#endif '__rp3d_classes__
