#ifndef __rp3d_collision__
#define __rp3d_collision__

namespace rp3d

type RaycastInfo
  ' Hit point in world-space coordinates
  as Vector3 worldPoint
  ' Surface normal at hit point in world-space coordinates
  as Vector3 worldNormal
  ' Fraction distance of the hit point between point1 and point2 of the ray
  ' The hit point "p" is such that p = point1 + hitFraction * (point2 - point1)
  as decimal hitFraction
  ' Mesh subpart index that has been hit (only used for triangles mesh and -1 otherwise)
  as long meshSubpart
  ' Hit triangle index (only used for triangles mesh and -1 otherwise)
  as long triangleIndex
  ' Pointer to the hit collision body
  as CollisionBody ptr body
  ' Pointer to the hit proxy collision shape
  as ProxyShape ptr proxyShape
end type

' This class represents a collision contact point between two bodies in the physics engine.
type ContactPoint as any
' a linked list of ContactPoint's
type ContactManifold as any
' This structure represents a single element of a linked list of contact manifolds
type ContactManifoldListElement as any 


type CollisionInfo
  ' Pointer to the first element of the linked-list 
  ' that contains contact manifolds
  as ContactManifoldListElement ptr manifoldListElement
  ' Pointer to the first body of the contact
  as CollisionBody ptr body1
  ' Pointer to the second body of the contact
  as CollisionBody ptr body2
  ' Pointer to the proxy shape of first body
  as const ProxyShape ptr proxyShape1
  ' Pointer to the proxy shape of second body
  as const ProxyShape ptr proxyShape2
end type

extern "C"

' #####################
' # optinal callbacks #
' #####################

type RaycastCB   as function (byref info as const RaycastInfo) as decimal
type OverlapCB   as sub      (byval collisionBody as CollisionBody ptr)
type CollisionCB as sub      (byref collisionInfo as const CollisionInfo)

' used by CollisionWorld Line/RayCast
declare sub      SetRaycastCallback(byval cb as RaycastCB)
declare function GetRaycastCallback() as RaycastCB

declare sub      SetOverlapCallback(byval cb as OverlapCB)
declare function GetOverlapCallback() as OverlapCB

declare sub      SetCollisionCallback(byval cb as CollisionCB)
declare function GetCollisionCallback() as CollisionCB


' ##############################
' # ContactManifoldListElement #
' ##############################
' Return the next element in the linked-list
declare function ContactManifoldListElementGetNext(byval cmle as ContactManifoldListElement ptr) as ContactManifoldListElement ptr
' Return the contact manifold
declare function ContactManifoldListElementGetContactManifold(byval cmle as ContactManifoldListElement ptr) as ContactManifold ptr

' ###################
' # ContactManifold #
' ###################
' Return a pointer to the previous element in the linked-list
declare function ContactManifoldGetPrevious(byval cm as ContactManifold ptr) as ContactManifold ptr
' Return a pointer to the next element in the linked-list
declare function ContactManifoldGetNext(byval cm as ContactManifold ptr) as ContactManifold ptr
' Return a pointer to the first proxy shape of the contact
declare function ContactManifoldGetShape1(byval cm as ContactManifold ptr) as ProxyShape ptr
' Return a pointer to the second proxy shape of the contact
declare function ContactManifoldGetShape2(byval cm as ContactManifold ptr) as ProxyShape ptr
' Return a pointer to the first body of the contact manifold
declare function ContactManifoldGetBody1(byval cm as ContactManifold ptr) as CollisionBody ptr
' Return a pointer to the second body of the contact manifold
declare function ContactManifoldGetBody2(byval cm as ContactManifold ptr) as CollisionBody ptr
' Return the number of contact points in the manifold
declare function ContactManifoldGetNbContactPoints(byval cm as ContactManifold ptr) as byte
' Return a pointer to the first contact point of the manifold
declare function ContactManifoldGetContactPoints(byval cm as ContactManifold ptr) as ContactPoint ptr


' ################
' # ContactPoint #
' ################
' Return the previous contact point in the linked list
declare function ContactPointGetPrevious(byval cpoint as ContactPoint ptr) as ContactPoint ptr
' Return the next contact point in the linked list
declare function ContactPointGetNext(byval cpoint as ContactPoint ptr) as ContactPoint ptr
' Return the normal vector of the contact
declare function ContactPointGetNormal(byval cpoint as ContactPoint ptr) as Vector3
' Return the contact point on the first proxy shape in the local-space of the proxy shape
declare function ContactPointGetLocalPointOnShape1(byval cpoint as ContactPoint ptr) byref as const Vector3
' Return the contact point on the second proxy shape in the local-space of the proxy shape
declare function ContactPointGetLocalPointOnShape2(byval cpoint as ContactPoint ptr) byref as const Vector3
' Return the cached penetration impulse
declare function ContactPointGetPenetrationImpulse(byval cpoint as ContactPoint ptr) as decimal
' Return true if the contact is a resting contact
declare function ContactPointGetIsRestingContact(byval cpoint as ContactPoint ptr) as boolean
' Return the penetration depth
declare function ContactPointGetPenetrationDepth(byval cpoint as ContactPoint ptr) as decimal

end extern

end namespace

#endif ' __rp3d_collision__
