#ifndef __rp3d_collisionbody__
#define __rp3d_collisionbody__

namespace rp3d

extern "C"

' ##############################
' # CollisionBody extends Body #
' ##############################
' This class represents a body that is able to collide with others bodies.
' This class inherits from the Body class.

' methods

' Set the type of the body
' The type of the body can either BT_STATIC, BT_KINEMATIC or BT_DYNAMIC as described bellow:

' BT_STATIC:    A static body has infinite mass, zero velocity but the position can be changed manually.
'               A static body does not collide with other static or kinematic bodies.

' BT_KINEMATIC: A kinematic body has infinite mass, the velocity can be changed manually
'               and its position is computed by the physics engine.
'               A kinematic body does not collide with other static or kinematic bodies.

' BT_DYNAMIC:   A dynamic body has non-zero mass, non-zero velocity determined by forces
'               and its position is determined by the physics engine.
'               A dynamic body can collide with other dynamic, static or kinematic bodies.
declare sub      CollisionBodySetType(byval cb as CollisionBody ptr, byval bt as eBodyType)
declare function CollisionBodyGetType(byval cb as CollisionBody ptr) as eBodyType

' Return the ID of the body
declare function CollisionBodyGetId alias "BodyGetId" (byval cb as CollisionBody ptr) as uinteger ' 32/64-bit

' Return whether or not the body is allowed to sleep
declare function CollisionBodyIsAllowedToSleep alias "BodyIsAllowedToSleep" (byval cb as CollisionBody ptr) as boolean
' Set whether or not the body is allowed to go to sleep
declare sub CollisionBodySetIsAllowedToSleep alias "BodySetIsAllowedToSleep" (byval cb as CollisionBody ptr, byval isAllowedToSleep as boolean)

' Set the variable to know whether or not the body is sleeping
declare sub CollisionBodySetIsSleeping alias "BodySetIsSleeping" (byval cb as CollisionBody ptr, byval isSleeping as boolean)
' Return whether or not the body is sleeping
declare function CollisionBodyIsSleeping alias "BodyIsSleeping" (byval cb as CollisionBody ptr) as boolean

' Return the user data attached to this body
declare function CollisionBodyGetUserData alias "BodyGetUserData" (byval cb as CollisionBody ptr) as any ptr
' Attach user data to this body
declare sub CollisionBodySetUserData alias "BodySetUserData" (byval cb as CollisionBody ptr, byval userData as any ptr)


' Set whether or not the body is active
declare sub      CollisionBodySetIsActive(byval cb as CollisionBody ptr, byval isActive as boolean)
' Return true if the body is active
declare function CollisionBodyIsActive alias "BodyIsActive" (byval cb as CollisionBody ptr) as boolean

declare sub      CollisionBodySetPosRotXYZ(byval cb as CollisionBody ptr, _
                                           byval px as decimal, byval py as decimal, byval pz as decimal, _
                                           byval rx as decimal, byval ry as decimal, byval rz as decimal)
declare sub      CollisionBodySetPosRot(byval cb as CollisionBody ptr, _
                                        byref position as const Vector3, _
                                        byref rotation as const Vector3)

declare function CollisionBodyGetTransform(byval cb as CollisionBody ptr) byref as const Transform

declare sub      CollisionBodySetTransform(byval cb as CollisionBody ptr, byref t as const Transform)

' Return true if a point is inside the collision body
declare function CollisionBodyTestPointInside(byval cb as CollisionBody ptr, _
                                              byref worldPoint as const Vector3) as boolean

' Raycast method with feedback information
declare function CollisionBodyRaycast(byval cb as CollisionBody ptr, _
                                      byref ray as const Ray, _
                                      byref raycastInfo as RaycastInfo) as boolean


' Test if the collision body overlaps with a given AABB
declare function CollisionBodyTestAABBOverlap(byval cb as CollisionBody ptr, _
                                              byref worldAABB as const AABB) as boolean

' Compute and return the AABB of the body by merging all proxy shapes AABBs
declare function CollisionBodyGetAABB(byval cb as CollisionBody ptr) as AABB

' Return the linked list of proxy shapes of that body
declare function CollisionBodyGetProxyShapesList(byval cb as CollisionBody ptr) as ProxyShape ptr

' Return the "const" linked list of proxy shapes of that body
declare function CollisionBodyGetProxyShapesListC(byval cb as CollisionBody ptr) as const ProxyShape ptr

' Return the world-space coordinates of a point given the local-space coordinates of the body
declare function CollisionBodyGetWorldPoint(byval cb as CollisionBody ptr, byref localPoint as const Vector3) as Vector3

' Return the body local-space coordinates of a point given in the world-space coordinates
declare function CollisionBodyGetLocalPoint(byval cb as CollisionBody ptr, byref worldPoint as const Vector3) as Vector3

' Return the world-space vector of a vector given in local-space coordinates of the body 
declare function CollisionBodyGetWorldVector(byval cb as CollisionBody ptr, byref localVector as const Vector3) as Vector3

' Return the body local-space coordinates of a vector given in the world-space coordinates
declare function CollisionBodyGetLocalVector(byval cb as CollisionBody ptr, byref worldVector as const Vector3) as Vector3

' Add a collision shape to the body.
' Note that you can share a collision shape between several bodies using the same 
' collision shape instance to when you add the shape to the different bodies.
' Do not forget to delete the collision shape you have created at the end of your program.

' This method will return a pointer to a new proxy shape.
' A proxy shape is an object that links a collision shape and a given body.
' You can use the returned proxy shape to get and set information 
' about the corresponding collision shape for that body.

' ProxyShape constructors
' Add a collision shape to the body.
declare function CollisionBodyAddCollisionShapePosRotXYZ(byval cb as CollisionBody ptr, _
                                                         byval cs as CollisionShape ptr, _
                                                         byval px as decimal, byval py as decimal, byval pz as decimal, _
                                                         byval rx as decimal, byval ry as decimal, byval rz as decimal) as ProxyShape ptr

declare function CollisionBodyAddCollisionShapePosRot(byval cb as CollisionBody ptr, _
                                                      byval cs as CollisionShape ptr, _
                                                      byref position as const Vector3, _
                                                      byref rotation as const Vector3) as ProxyShape ptr

declare function CollisionBodyAddCollisionShape(byval cb as CollisionBody ptr, _
                                                byval cs as CollisionShape ptr, _
                                                byref t  as const Transform) as ProxyShape ptr

declare function CollisionBodyAddCollisionShapeAtCenter(byval cb as CollisionBody ptr, _
                                                        byval cs as CollisionShape ptr) as ProxyShape ptr
' Remove a collision shape from the body
declare sub      CollisionBodyRemoveCollisionShape(byval cb as CollisionBody ptr, byval proxyShape as const ProxyShape ptr)
end extern

end namespace


#endif ' __rp3d_collisionbody__
