#ifndef __rp3d_joint__
#define __rp3d_joint__

namespace rp3d

extern "C"

' #########
' # Joint #
' #########
' This abstract class represents a joint between two bodies.

' note: joints are created by the DynamicsWorld see: "rp3d_dynamicsworld.bi"

' methods

' Return the type of the constraint
declare function JointGetType(byval j as Joint ptr) as eJointType
' Return the id of the joint
declare function JointGetId(byval j as Joint ptr) as ulong
' Return true if the constraint is active
declare function JointIsActive(byval j as Joint ptr) as boolean
' Return the reference to the body 1
declare function JointGetBody1(byval j as Joint ptr) as RigidBody ptr 
' Return the reference to the body 2
declare function JointGetBody2(byval j as Joint ptr) as RigidBody ptr
' Return true if the collision between the two bodies of the joint is enabled
declare function JointIsCollisionEnabled(byval j as Joint ptr) as boolean
' Return the user data attached to this joint
declare function JointGetUserData(byval j as Joint ptr) as any ptr
' Attach user data to this joint
declare sub      JointSetUserData(byval j as Joint ptr, byval userData as any ptr)

' ############################
' # FixedJoint extends Joint #
' ############################
 ' This class represents a fixed joint that is used to forbid
 ' any translation or rotation between two bodies.
#define FixedJointGetType  JointGetType
#define FixedJointGetId    JointGetId
#define FixedJointIsActive JointIsActive
#define FixedJointGetBody1 JointGetBody1
#define FixedJointGetBody2 JointGetBody2

' ####################################
' # BallAndSocketJoint extends Joint #
' ####################################
' This class represents a ball-and-socket joint 
' that allows arbitrary rotation between two bodies.
' This joint has three degrees of freedom.
' It can be used to create a chain of bodies for instance.
#define BallAndSocketJointGetType  JointGetType
#define BallAndSocketJointGetId    JointGetId
#define BallAndSocketJointIsActive JointIsActive
#define BallAndSocketJointGetBody1 JointGetBody1
#define BallAndSocketJointGetBody2 JointGetBody2


' ############################
' # HingeJoint extends Joint #
' ############################
' This class represents a hinge joint that allows 
' arbitrary rotation between two bodies around a single axis.
' This joint has one degree of freedom.
' It can be useful to simulate doors or pendulumns.

' methods

' Return true if the limits or the joint are enabled
declare function HingeJointIsLimitEnabled(byval hj as HingeJoint ptr) as boolean
' Enable/Disable the limits of the joint
declare sub      HingeJointEnableLimit(byval hj as HingeJoint ptr, byval isLimitEnabled as boolean)

' Return true if the motor of the joint is enabled
declare function HingeJointIsMotorEnabled(byval hj as HingeJoint ptr) as boolean
' Enable/Disable the motor of the joint
declare sub      HingeJointEnableMotor(byval hj as HingeJoint ptr, byval isMotorEnabled as boolean)

' Return the minimum angle limit
declare function HingeJointGetMinAngleLimit(byval hj as HingeJoint ptr) as decimal
' Set the minimum angle limit
declare sub      HingeJointSetMinAngleLimit(byval hj as HingeJoint ptr, byval lowerLimit as decimal)

' Return the maximum angle limit
declare function HingeJointGetMaxAngleLimit(byval hj as HingeJoint ptr) as decimal
' Set the maximum angle limit
declare sub      HingeJointSetMaxAngleLimit(byval hj as HingeJoint ptr, byval upperLimit as decimal)

' Return the motor speed
declare function HingeJointGetMotorSpeed(byval hj as HingeJoint ptr) as decimal
' Set the motor speed
declare sub      HingeJointSetMotorSpeed(byval hj as HingeJoint ptr, byval motorSpeed as decimal)

' Return the maximum motor torque
declare function HingeJointGetMaxMotorTorque(byval hj as HingeJoint ptr) as decimal
' Set the maximum motor torque
declare sub      HingeJointSetMaxMotorTorque(byval hj as HingeJoint ptr, byval maxMotorTorque as decimal)

' Return the intensity of the current torque applied for the joint motor
declare function HingeJointGetMotorTorque(byval hj as HingeJoint ptr, byval timeStep as decimal) as decimal

#define HingeJointGetType  JointGetType
#define HingeJointGetId    JointGetId
#define HingeJointIsActive JointIsActive
#define HingeJointGetBody1 JointGetBody1
#define HingeJointGetBody2 JointGetBody2

' #############################
' # SliderJoint extends Joint #
' #############################
' This class represents a slider joint.
' This joint has a one degree of freedom.
' It only allows relative translation of the bodies
' along a single direction and no rotation.

' methods

' Return true if the limits or the joint are enabled
declare function SliderJointIsLimitEnabled(byval sj as SliderJoint ptr) as boolean
' Enable/Disable the limits of the joint
declare sub      SliderJointEnableLimit(byval sj as SliderJoint ptr, byval isLimitEnabled as boolean)
' Return true if the motor of the joint is enabled
declare function SliderJointIsMotorEnabled(byval sj as SliderJoint ptr) as boolean
' Enable/Disable the motor of the joint
declare sub      SliderJointEnableMotor(byval sj as SliderJoint ptr, byval isMotorEnabled as boolean)
' Return the current translation value of the joint
declare function SliderJointGetTranslation(byval sj as SliderJoint ptr) as decimal
' Return the minimum translation limit
declare function SliderJointGetMinTranslationLimit(byval sj as SliderJoint ptr) as decimal
' Set the minimum translation limit
declare sub      SliderJointSetMinTranslationLimit(byval sj as SliderJoint ptr, byval lowerLimit as decimal)
' Return the maximum translation limit
declare function SliderJointGetMaxTranslationLimit(byval sj as SliderJoint ptr) as decimal
' Set the maximum translation limit
declare sub      SliderJointSetMaxTranslationLimit(byval sj as SliderJoint ptr, byval upperLimit as decimal)
' Return the motor speed
declare function SliderJointGetMotorSpeed(byval sj as SliderJoint ptr) as decimal
' Set the motor speed
declare sub      SliderJointSetMotorSpeed(byval sj as SliderJoint ptr, byval motorSpeed as decimal)
' Return the maximum motor force
declare function SliderJointGetMaxMotorForce(byval sj as SliderJoint ptr) as decimal
' Set the maximum motor force
declare sub      SliderJointSetMaxMotorForce(byval sj as SliderJoint ptr, byval maxMotorForce as decimal)
' Return the intensity of the current force applied for the joint motor
declare function SliderJointGetMotorForce(byval sj as SliderJoint ptr, byval timeStep as decimal) as decimal
#define SliderJointGetType  JointGetType
#define SliderJointGetId    JointGetId
#define SliderJointIsActive JointIsActive
#define SliderJointGetBody1 JointGetBody1
#define SliderJointGetBody2 JointGetBody2


end extern

end namespace

#endif ' __rp3d_joint__
