#ifndef __rp3d_jointinfo__
#define __rp3d_jointinfo__

namespace rp3d

' Enumeration for the type of a constraint
enum eJointType
  BALLSOCKETJOINT = 0
  SLIDERJOINT
  HINGEJOINT
  FIXEDJOINT
end enum

enum eJointsPositionCorrectionTechnique
  BAUMGARTE_JOINTS = 0
  NON_LINEAR_GAUSS_SEIDEL
end enum  
enum eContactsPositionCorrectionTechnique
  BAUMGARTE_CONTACTS=0
  SPLIT_IMPULSES
end enum

' This structure represents a single element of a linked list of joints
type JointListElement
  ' Pointer to the actual joint
  as Joint ptr joint
  ' Next element of the list
  as JointListElement ptr next
end type
  
' This structure is used to gather 
' the information needed to create a joint.
type JointInfo
  public:
  declare constructor
  declare constructor (byval constraintType as eJointType)
  declare constructor (byval rigidBody1 as RigidBody ptr, _
                       byval rigidBody2 as RigidBody ptr,_
                       byval constraintType as eJointType)
  as RigidBody ptr body1
  as RigidBody ptr body2
  as eJointType jointType
  as eJointsPositionCorrectionTechnique positionCorrectionTechnique
  as boolean isCollisionEnabled
end type
constructor JointInfo
end constructor
constructor JointInfo(byval constraintType as eJointType)
  this.jointType=constraintType
  this.positionCorrectionTechnique = eJointsPositionCorrectionTechnique.NON_LINEAR_GAUSS_SEIDEL
  this.isCollisionEnabled = true
end constructor
constructor JointInfo(byval rigidBody1 as RigidBody ptr, _
                      byval rigidBody2 as RigidBody ptr,_
                      byval constraintType as eJointType)
  this.body1 = rigidBody1
  this.body2 = rigidBody2
  this.jointType = constraintType
  this.positionCorrectionTechnique = eJointsPositionCorrectionTechnique.NON_LINEAR_GAUSS_SEIDEL
  this.isCollisionEnabled = true
end constructor

' This structure is used to gather the information needed to create a fixed joint.
' This structure will be used to create the actual fixed joint.
type FixedJointInfo extends JointInfo
  public :
  declare constructor(byval Body1 as RigidBody ptr, _
                      byval Body2 as RigidBody ptr, _
                      byref initAnchorPointWorldSpace as const Vector3)
  declare constructor(byval Body1 as RigidBody ptr, _
                      byval Body2 as RigidBody ptr, _
                      byval initAnchorPointWorldSpaceX as single, _
                      byval initAnchorPointWorldSpaceY as single, _
                      byval initAnchorPointWorldSpaceZ as single)

  ' Anchor point (in world-space coordinates)
  as Vector3 anchorPointWorldSpace
end type
constructor FixedJointInfo(byval Body1 as RigidBody ptr, _
                           byval Body2 as RigidBody ptr, _
                           byref initAnchorPointWorldSpace as const Vector3)
  base(Body1, Body2, eJointType.FIXEDJOINT)
  this.anchorPointWorldSpace = initAnchorPointWorldSpace
end constructor
constructor FixedJointInfo(byval Body1 as RigidBody ptr, _
                           byval Body2 as RigidBody ptr, _
                           byval initAnchorPointWorldSpaceX as single, _
                           byval initAnchorPointWorldSpaceY as single, _
                           byval initAnchorPointWorldSpaceZ as single)
  base(Body1, Body2, eJointType.FIXEDJOINT)
  this.anchorPointWorldSpace = type(initAnchorPointWorldSpaceX, _
                                    initAnchorPointWorldSpaceY, _
                                    initAnchorPointWorldSpaceZ)
end constructor


' This structure is used to gather the information needed to create a ball-and-socket joint.
' This structure will be used to create the actual ball-and-socket joint.
type BallAndSocketJointInfo extends JointInfo
  public :
  declare constructor(byval Body1 as RigidBody ptr, _
                      byval Body2 as RigidBody ptr, _
                      byref initAnchorPointWorldSpace as const Vector3)
  declare constructor(byval Body1 as RigidBody ptr, _
                      byval Body2 as RigidBody ptr, _
                      byval initAnchorPointWorldSpaceX as single, _
                      byval initAnchorPointWorldSpaceY as single, _
                      byval initAnchorPointWorldSpaceZ as single)
  ' Anchor point (in world-space coordinates)
  as Vector3 anchorPointWorldSpace
end type
constructor BallAndSocketJointInfo(byval Body1 as RigidBody ptr, _
                                   byval Body2 as RigidBody ptr, _
                                   byref initAnchorPointWorldSpace as const Vector3)
  base(Body1, Body2, eJointType.BALLSOCKETJOINT)
  this.anchorPointWorldSpace = initAnchorPointWorldSpace
end constructor
constructor BallAndSocketJointInfo(byval Body1 as RigidBody ptr, _
                                   byval Body2 as RigidBody ptr, _
                                   byval initAnchorPointWorldSpaceX as single, _
                                   byval initAnchorPointWorldSpaceY as single, _
                                   byval initAnchorPointWorldSpaceZ as single)
  base(Body1, Body2, eJointType.BALLSOCKETJOINT)
  this.anchorPointWorldSpace = type(initAnchorPointWorldSpaceX, _
                                    initAnchorPointWorldSpaceY, _
                                    initAnchorPointWorldSpaceZ)  
end constructor
' This structure is used to gather the information needed to create a hinge joint.
' This structure will be used to create the actual hinge joint.
type HingeJointInfo extends JointInfo
  public:
  ' Constructor without limits and without motor
  declare constructor(byval Body1                     as RigidBody ptr, _
                      byval Body2                     as RigidBody ptr, _
                      byref initAnchorPointWorldSpace as const Vector3, _
                      byref initRotationAxisWorld     as const Vector3)
  ' Constructor with limits but without motor
  declare constructor(byval Body1                     as RigidBody ptr, _
                      byval Body2                     as RigidBody ptr, _
                      byref initAnchorPointWorldSpace as const Vector3, _
                      byref initRotationAxisWorld     as const Vector3, _
                      byval initMinAngleLimit         as decimal, _
                      byval initMaxAngleLimit         as decimal)
  ' Constructor with limits and motor                      
  declare constructor(byval Body1                     as RigidBody ptr, _
                      byval Body2                     as RigidBody ptr, _
                      byref initAnchorPointWorldSpace as const Vector3, _
                      byref initRotationAxisWorld     as const Vector3, _
                      byval initMinAngleLimit         as decimal, _
                      byval initMaxAngleLimit         as decimal, _
                      byval initMotorSpeed            as decimal, _
                      byval initMaxMotorTorque        as decimal) 
  ' Anchor point (in world-space coordinates)
  as Vector3 anchorPointWorldSpace
  ' Hinge rotation axis (in world-space coordinates)
  as Vector3 rotationAxisWorld
  ' True if the hinge joint limits are enabled
  as boolean isLimitEnabled
  ' True if the hinge joint motor is enabled
  as boolean isMotorEnabled
  ' Minimum allowed rotation angle (in radian) if limits are enabled.
  ' The angle must be in the range [-2*pi, 0]
  as decimal minAngleLimit
  ' Maximum allowed rotation angle (in radian) if limits are enabled.
  ' The angle must be in the range [0, 2*pi]
  as decimal maxAngleLimit
  ' Motor speed (in radian/second)
  as decimal motorSpeed
  ' Maximum motor torque (in Newtons * meters) that can be applied to reach
  ' to desired motor speed
  as decimal maxMotorTorque
end type
constructor HingeJointInfo(byval Body1 as RigidBody ptr, _
                           byval Body2 as RigidBody ptr, _
                           byref initAnchorPointWorldSpace as const Vector3, _
                           byref initRotationAxisWorld     as const Vector3)
  base(Body1, Body2, eJointType.HINGEJOINT)
  this.anchorPointWorldSpace = initAnchorPointWorldSpace
  this.rotationAxisWorld     = initRotationAxisWorld
  this.isLimitEnabled        = false
  this.isMotorEnabled        = false 
  this.minAngleLimit         = -1
  this.maxAngleLimit         = 1
  motorSpeed                 = 0
  maxMotorTorque             = 0
end constructor

constructor HingeJointInfo(byval Body1 as RigidBody ptr, _
                           byval Body2 as RigidBody ptr, _
                           byref initAnchorPointWorldSpace as const Vector3, _
                           byref initRotationAxisWorld     as const Vector3, _
                           byval initMinAngleLimit         as decimal, _
                           byval initMaxAngleLimit         as decimal)  
  base(Body1, Body2, eJointType.HINGEJOINT)
  this.anchorPointWorldSpace = initAnchorPointWorldSpace
  this.rotationAxisWorld = initRotationAxisWorld
  this.isLimitEnabled = true
  this.isMotorEnabled = false 
  this.minAngleLimit = initMinAngleLimit
  this.maxAngleLimit = initMaxAngleLimit
  motorSpeed = 0
  maxMotorTorque =0
end constructor

constructor HingeJointInfo(byval Body1 as RigidBody ptr, _
                           byval Body2 as RigidBody ptr, _
                           byref initAnchorPointWorldSpace as const Vector3, _
                           byref initRotationAxisWorld     as const Vector3, _
                           byval initMinAngleLimit         as decimal, _
                           byval initMaxAngleLimit         as decimal, _
                           byval initMotorSpeed            as decimal, _
                           byval initMaxMotorTorque        as decimal) 

  base(Body1, Body2, eJointType.HINGEJOINT)
  this.anchorPointWorldSpace = initAnchorPointWorldSpace
  this.rotationAxisWorld     = initRotationAxisWorld
  this.isLimitEnabled        = true
  this.isMotorEnabled        = false 
  this.minAngleLimit         = initMinAngleLimit
  this.maxAngleLimit         = initMaxAngleLimit
  this.motorSpeed            = initMotorSpeed
  this.maxMotorTorque        = initMaxMotorTorque
end constructor

' This structure is used to gather the information needed to create a slider joint.
' This structure will be used to create the actual slider joint.
type SliderJointInfo extends JointInfo
  public:
  ' Constructor without limits and without motor
  declare constructor (byval Body1 as RigidBody ptr, _
                       byval Body2 as RigidBody ptr, _
                       byref initAnchorPointWorldSpace as const Vector3, _
                       byref initSliderAxisWorldSpace  as const Vector3)
  ' Constructor with limits and no motor
  declare constructor (byval Body1 as RigidBody ptr, _
                       byval Body2 as RigidBody ptr, _
                       byref initAnchorPointWorldSpace as const Vector3, _
                       byref initSliderAxisWorldSpace  as const Vector3, _
                       byval initMinTranslationLimit   as decimal, _
                       byval initMaxTranslationLimit   as decimal)
  ' Constructor with limits and motor
  declare constructor (byval Body1 as RigidBody ptr, _
                       byval Body2 as RigidBody ptr, _
                       byref initAnchorPointWorldSpace as const Vector3, _
                       byref initSliderAxisWorldSpace  as const Vector3, _
                       byval initMinTranslationLimit   as decimal, _
                       byval initMaxTranslationLimit   as decimal, _
                       byval initMotorSpeed            as decimal, _
                       byval initMaxMotorForce         as decimal)
  ' Anchor point (in world-space coordinates)
  as Vector3 anchorPointWorldSpace
  ' Slider axis (in world-space coordinates)
  as Vector3 sliderAxisWorldSpace
  ' True if the slider limits are enabled
  as boolean isLimitEnabled
  ' True if the slider motor is enabled
  as boolean isMotorEnabled
  ' Mininum allowed translation if limits are enabled
  as decimal minTranslationLimit
  ' Maximum allowed translation if limits are enabled
  as decimal maxTranslationLimit
  ' Motor speed
  as decimal motorSpeed
  ' Maximum motor force (in Newtons) that can be applied to reach to desired motor speed
  as decimal maxMotorForce
end type
constructor SliderJointInfo(byval Body1 as RigidBody ptr, _
                            byval Body2 as RigidBody ptr, _
                            byref initAnchorPointWorldSpace as const Vector3, _
                            byref initSliderAxisWorldSpace  as const Vector3)
  base(Body1, Body2, eJointType.SLIDERJOINT)
  this.anchorPointWorldSpace = initAnchorPointWorldSpace
  this.sliderAxisWorldSpace  = initSliderAxisWorldSpace
  this.isLimitEnabled        = false
  this.isMotorEnabled        = false
  this.minTranslationLimit   = -1.0
  this.maxTranslationLimit   = 1.0
  this.motorSpeed            = 0
  this.maxMotorForce         = 0
end constructor
constructor SliderJointInfo(byval Body1 as RigidBody ptr, _
                            byval Body2 as RigidBody ptr, _
                            byref initAnchorPointWorldSpace as const Vector3, _
                            byref initSliderAxisWorldSpace  as const Vector3, _
                            byval initMinTranslationLimit   as decimal, _
                            byval initMaxTranslationLimit   as decimal)
  base(Body1, Body2, eJointType.SLIDERJOINT)
  this.anchorPointWorldSpace = initAnchorPointWorldSpace
  this.sliderAxisWorldSpace = initSliderAxisWorldSpace
  this.isLimitEnabled = true
  this.isMotorEnabled = false
  this.minTranslationLimit=initMinTranslationLimit
  this.maxTranslationLimit=initMaxTranslationLimit
  this.motorSpeed=0
  this.maxMotorForce=0
end constructor
constructor SliderJointInfo(byval Body1                     as RigidBody ptr, _
                            byval Body2                     as RigidBody ptr, _
                            byref initAnchorPointWorldSpace as const Vector3, _
                            byref initSliderAxisWorldSpace  as const Vector3, _
                            byval initMinTranslationLimit   as decimal, _
                            byval initMaxTranslationLimit   as decimal, _
                            byval initMotorSpeed            as decimal, _
                            byval initMaxMotorForce         as decimal)
  base(Body1, Body2, eJointType.SLIDERJOINT)
  anchorPointWorldSpace = initAnchorPointWorldSpace
  sliderAxisWorldSpace = initSliderAxisWorldSpace
  isLimitEnabled = true
  isMotorEnabled = true
  minTranslationLimit = initMinTranslationLimit
  maxTranslationLimit = initMaxTranslationLimit
  motorSpeed = initMotorSpeed
  maxMotorForce = initMaxMotorForce
end constructor


end namespace

#endif ' __rp3d_jointinfo__
