#ifndef __rp3d_math__
#define __rp3d_math__

namespace rp3d

const as decimal PI  = 3.14159265
const as decimal PI2 = 6.28318530

type Vector3
  as decimal x,y,z
end type

type Quaternion
  as decimal x,y,z,w
end type

type Matrix3x3
  as Vector3 Rows(2)
end type  

type Transform
  as Vector3    Position
  as Quaternion Orientation
end type

type AABB
  ' Minimum and Maximum world coordinates of the AABB on the x,y and z axis
  as Vector3 MinCoordinates
  as Vector3 MaxCoordinates
end type

type Ray
  'First point of the ray (origin) in world-space
  as Vector3 point1
  ' Second point of the ray in world-space
  as Vector3 point2
  ' Maximum fraction value
  as decimal maxFraction
end type

extern "C"

' ##############
' # Quaternion #
' ##############
' This class represents a quaternion.
' We use the notation:  q = (x*i, y*j, z*k, w) to represent a quaternion.

' constructors
declare function QuaternionIdentity() as Quaternion
declare function QuaternionFromEulerAnglesXYZ(byval rx as decimal, _
                                              byval ry as decimal, _
                                              byval rz as decimal) as Quaternion
declare function QuaternionFromEulerAngles(byref rot as const Vector3) as Quaternion

declare function QuaternionFromMatrix(byref m as const Matrix3x3) as Quaternion

' methods
declare sub QuaternionInitWithEulerAnglesXYZ(byref quat as Quaternion, _
                                             byval rx as decimal, _
                                             byval ry as decimal, _
                                             byval rz as decimal)
declare sub QuaternionInitWithEulerAngles(byref quat as Quaternion, _
                                          byref rot  as const Vector3)

declare sub QuaternionInitSLerp(byref quat as Quaternion, _
                                byref a as const Quaternion, _
                                byref b as const Quaternion, _
                                byval t as decimal)

declare sub QuaternionGetRotationAngleAxisXYZ(byref quat as const Quaternion, _
                                              byref angle as decimal, _
                                              byref rx as decimal, _
                                              byref ry as decimal, _
                                              byref rz as decimal)
declare sub QuaternionGetRotationAngleAxis(byref quat as const Quaternion, _
                                           byref angle as decimal, _
                                           byref axis as Vector3)
' a*=b
declare sub QuaternionMulWith(byref a as Quaternion, _
                              byref b as const Quaternion)

' Set the quaternion to zero
declare sub      QuaternionSetToZero(byref q as Quaternion)
' Set to the identity quaternion
declare sub      QuaternionSetToIdentity(byref q as Quaternion)
' Return the length of the quaternion
declare function QuaternionGetLength(byref q as Quaternion) as decimal
' Return the square of the length of the quaternion
declare function QuaternionGetLengthSquare(byref q as Quaternion) as decimal
' Normalize the quaternion
declare sub      QuaternionNormalize(byref q as Quaternion)
declare function QuaternionNormalized(byref q as Quaternion) as Quaternion
' Inverse the quaternion
declare sub      QuaternionInverse(byref q as Quaternion)
' Return the unit quaternion
declare function QuaternionGetUnit(byref q as const Quaternion) as Quaternion
' Return the conjugate quaternion
declare function QuaternionGetConjugate(byref q as const Quaternion) as Quaternion
' Return the inverse of the quaternion
declare function QuaternionGetInverse(byref q as const Quaternion) as Quaternion
' Return the orientation matrix corresponding to this quaternion
declare function QuaternionGetMatrix(byref q as const Quaternion) as Matrix3x3
' Dot product between two quaternions
declare function QuaternionGetDot(byref a as const Quaternion, byref b as const Quaternion) as decimal
' Compute the spherical linear interpolation between two quaternions
declare function QuaternionSlerp(byref a as const Quaternion, byref b as const Quaternion, byval t as decimal) as Quaternion
' Overloaded operator for addition with assignment
declare sub      QuaternionAddToThis(byref q as Quaternion, byref a as const Quaternion)
' Overloaded operator for substraction with assignment
declare sub      QuaternionSubFromThis(byref q as Quaternion, byref a as const Quaternion)
' Overloaded operator for the multiplication with a constant
declare sub      QuaternionMulThis(byref q as Quaternion, byval n as decimal)
' Overloaded operator for the addition
declare function QuaternionAdd(byref a as const Quaternion, byref b as const Quaternion) as Quaternion
' Overloaded operator for the substraction
declare function QuaternionSub(byref a as const Quaternion, byref b as const Quaternion) as Quaternion
' Overloaded operator for the multiplication
declare function QuaternionMul(byref a as const Quaternion, byref b as const Quaternion) as Quaternion
' Overloaded operator for the multiplication with a vector
declare function QuaternionMulVector(byref q as const Quaternion, byref v as const Vector3) as Vector3

' #############
' # Transform #
' #############
' This class represents a position and an orientation in 3D.
' It can also be seen as representing a translation and a rotation.
 
' constructors
declare function TransformFromFromPosMatrix(byref position as const Vector3, _
                                            byref orientation as const Matrix3x3) as Transform
declare function TransformFromPosRotXYZ(byval px as decimal, byval py as decimal, byval pz as decimal, _
                                        byval rx as decimal, byval ry as decimal, byval rz as decimal) as Transform
declare function TransformFromPosRot(byref position as const Vector3, byref rotation as const Vector3) as Transform

' methods
declare function TransformGetPosition(byref t as const Transform) byref as const Vector3
declare sub      TransformGetPositionXYZ(byref t as const Transform, byref px as decimal, byref py as decimal, byref pz as decimal)

declare sub      TransformSetPosition(byref t as Transform, byref position as const Vector3)
declare sub      TransformSetPositionXYZ(byref t as Transform, byval px as decimal, byval py as decimal, byval pz as decimal)

declare function TransformGetOrientation(byref t as const Transform) byref as const Quaternion  

declare sub      TransformSetOrientation(byref t as Transform, byref o as const Quaternion)
declare sub      TransformSetOrientationFromEulerXYZ(byref t as Transform, byval rx as decimal, byval ry as decimal, byval rz as decimal)

declare sub      TransformSetOpenGLMatrix(byref t as Transform, byval matri4x4 as decimal ptr)
declare sub      TransformGetOpenGLMatrix(byref t as Transform, byval matri4x4 as decimal ptr)

' Set the transform to the identity transform
declare sub      TransformSetToIdentity(byref t as Transform)
' Return the inverse of the transform
declare function TransformGetInverse(byref t as const Transform) as Transform
' Return an interpolated transform
declare function TransformInterpolate(byref a as const Transform, byref b as const Transform, byval t as decimal) as Transform
' Return the identity transform
declare function TransformGetIdentity(byref t as Transform) as Transform
' Return the transformed vector
declare function TransformMulVector(byref t as const Transform, byref v as const Vector3) as Vector3
' Operator of multiplication of a transform with another one
declare function TransformMul(byref a as const Transform, byref b as const Transform) as Transform


' #############
' # Matrix3x3 #
' #############
' constructor
declare function CreateMatrix(byval a1 as decimal, byval a2 as decimal, byval a3 as decimal, _
                              byval b1 as decimal, byval b2 as decimal, byval b3 as decimal, _
                              byval c1 as decimal, byval c2 as decimal, byval c3 as decimal) as Matrix3x3
' Return the 3x3 identity matrix
declare function CreateIdentityMatrix() as Matrix3x3

' methods

' Set the matrix to the identity matrix
declare sub      MatrixSetToIdentity(byref m as Matrix3x3)
' Set the matrix to zero
declare sub      MatrixSetToZero(byref m as Matrix3x3)
' Return a column
declare function MatrixGetColumn(byref m as const Matrix3x3, byval index as long) as Vector3
' Return a row
declare function MatrixGetRow(byref m as const Matrix3x3, byval index as long) as Vector3
' Return the transpose matrix
declare function MatrixGetTranspose(byref m as const Matrix3x3) as Matrix3x3
' Return the determinant of the matrix
declare function MatrixGetDeterminant(byref m as const Matrix3x3) as decimal
' Return the trace of the matrix
declare function MatrixGetTrace(byref m as const Matrix3x3) as decimal
' Return the inverse matrix
declare function MatrixGetInverse(byref m as const Matrix3x3) as Matrix3x3
' Return the matrix with absolute values
declare function MatrixGetAbsoluteMatrix(byref m as const Matrix3x3) as Matrix3x3
' m=a+b Overloaded operator for addition
declare function MatrixAdd(byref a as const Matrix3x3, byref b as const Matrix3x3) as Matrix3x3
' m=a-b Overloaded operator for substraction
declare function MatrixSub(byref a as const Matrix3x3, byref b as const Matrix3x3) as Matrix3x3
' m=-a Overloaded operator for the negative of the matrix
declare function MatrixNeg(byref a as const Matrix3x3) as Matrix3x3
' m*=n Overloaded operator for multiplication with a matrix
declare sub      MatrixMulThis(byref m as Matrix3x3, byval n as decimal)
' m=a*b Overloaded operator for matrix multiplication
declare function MatrixMul(byref a as const Matrix3x3, byref b as const Matrix3x3) as Matrix3x3
' v'=m*v Overloaded operator for multiplication with a vector
declare function MatrixMulVector(byref m as const Matrix3x3, byref v as const Vector3) as Vector3

' ########
' # AABB #
' ########
' Constructor
declare function CreateAABB(byref vmin as const Vector3, byref vmax as const Vector3) as AABB
' Create and return an AABB for a triangle
declare function CreateAABBForTriangle(byval trianglePoints as const Vector3 ptr) as AABB

' methods
' Return the center point
declare function AABBGetCenter(byref a as const AABB) as Vector3
' Return the minimum coordinates of the AABB
declare function AABBGetMin(byref a as const AABB) byref as const Vector3
' Set the minimum coordinates of the AABB
declare sub      AABBSetMin(byref a as AABB, byref vmin as const Vector3)
' Return the maximum coordinates of the AABB
declare function AABBGetMax(byref a as const AABB) byref as const Vector3
' Set the maximum coordinates of the AABB
declare sub      AABBSetMax(byref a as AABB, byref vmax as const Vector3)
' Return the size of the AABB in the three dimension x, y and z
declare function AABBGetExtent(byref a as const AABB) as Vector3
' Inflate each side of the AABB by a given size
declare sub      AABBInflate(byref a as AABB, byval dx as decimal, byval dy as decimal, byval dz as decimal)
' Return true if the current AABB is overlapping with the AABB in argument
declare function AABBTestCollision(byref a as const AABB, byref b as const AABB) as boolean
' Return the volume of the AABB
declare function AABBGetVolume(byref a as const AABB) as decimal
' Merge the AABB in parameter with the current one
declare sub      AABBMergeWithAABB(byref a as AABB, byref b as const AABB)
' Replace the current AABB with a new AABB that is the union of two AABBs in parameters
declare sub      AABBMergeTwoAABBs(byref a as AABB, byref b as const AABB, byref c as const AABB)
' Return true if the current AABB contains the AABB given in parameter
declare function AABBContains(byref a as const AABB, byref b as const AABB) as boolean
' Return true if a point is inside the AABB
declare function AABBContainsVector(byref a as const AABB, byref v as const Vector3) as boolean
' Return true if the AABB of a triangle intersects the AABB
declare function AABBTestCollisionTriangleAABB(byref a as const AABB, byval trianglePoints as const Vector3 ptr) as boolean
' Return true if the ray intersects the AABB
declare function AABBTestRayIntersect(byref a as const AABB, byref r as const Ray) as boolean


end extern

end namespace

#endif ' __rp3d_math__
