#ifndef __rp3d_proxyshape__
#define __rp3d_proxyshape__

namespace rp3d

' ##############
' # ProxyShape #
' ##############
' Class ProxyShape
' The CollisionShape instances are supposed to be unique for memory optimization.
' For instance, consider two rigid bodies with the same sphere collision shape.
' In this situation, we will have a unique instance of SphereShape but we need 
' to differentiate between the two instances during the collision detection.
' They do not have the same position in the world and they do not belong to the same rigid body.
' The ProxyShape class is used for that purpose by attaching a rigid body with one of its collision shape.
' A body can have multiple proxy shapes (one for each collision shape attached to the body).

extern "C"

' methods

' Return the parent body
declare function ProxyShapeGetBody(byval proxy as ProxyShape ptr) as CollisionBody ptr
' Return the mass of the collision shape
declare function ProxyShapeGetMass(byval proxy as ProxyShape ptr) as decimal
' Return a pointer to the user data attached to this body
declare function ProxyShapeGetUserData(byval proxy as ProxyShape ptr) as any ptr
' Attach user data to this body
declare sub      ProxyShapeSetUserData(byval proxy as ProxyShape ptr, byval userData as any ptr)
' Return the local to parent body transform
declare function ProxyShapeGetLocalToBodyTransform(byval proxy as ProxyShape ptr) byref as const Transform
' Set the local to parent body transform
declare sub      ProxyShapeSetLocalToBodyTransform(byval proxy as ProxyShape ptr,byref t as const Transform)
' Return the local to world transform
declare function ProxyShapeGetLocalToWorldTransform(byval proxy as ProxyShape ptr) as const Transform
' Return the AABB of the proxy shape in world-space
declare function ProxyShapeGetWorldAABB(byval proxy as ProxyShape ptr) as const AABB
' Test if the proxy shape overlaps with a given AABB
declare function ProxyShapeTestAABBOverlap(byval proxy as ProxyShape ptr,byref worldAABB as const AABB) as boolean
' Return true if a point is inside the collision shape
declare function ProxyShapeTestPointInside(byval proxy as ProxyShape ptr, byref worldPoint as const Vector3) as boolean
' Raycast method with feedback information
declare function ProxyShapeLinecast(byval proxy as ProxyShape ptr, _
                                    byref startPoint as const Vector3, _
                                    byref endPoint   as const Vector3, _
                                    byref raycastInfo as RaycastInfo) as boolean
declare function ProxyShapeRaycast(byval proxy as ProxyShape ptr, _
                                   byref ray as const Ray, _
                                   byref raycastInfo as RaycastInfo) as boolean

' Set the collision bits mask
' Bits mask used to state which collision categories this shape cancollide with.
' This value is &HFFFF by default.
' It means that this proxy shape will collide with every collision categories by default.
declare sub      ProxyShapeSetCollideWithMaskBits(byval proxy as ProxyShape ptr, _
                                                  byval collideWithMaskBits as ushort)
' Return the collision bits mask
declare function ProxyShapeGetCollideWithMaskBits(byval proxy as ProxyShape ptr) as ushort

' Set the collision category bits
' Bits used to define the collision category of this shape.
' You can set a single bit to one to define a category value for this shape.
' This value is one (&H0001) by default.
' This variable can be used together with the mCollideWithMaskBits variable
' so that given categories of shapes collide with each other
' and do not collide with other categories.
declare sub ProxyShapeSetCollisionCategoryBits(byval proxy as ProxyShape ptr, _
                                               byval collisionCategoryBits as ushort)
' Return the collision category bits
declare function ProxyShapeGetCollisionCategoryBits(byval proxy as ProxyShape ptr) as ushort
                                                    
' Return the next proxy shape in the linked list of proxy shapes
declare function ProxyShapeGetNext(byval proxy as ProxyShape ptr) as ProxyShape ptr
' Return the next const proxy shape in the linked list of proxy shapes
declare function ProxyShapeGetNextC(byval proxy as ProxyShape ptr) as const ProxyShape ptr

end extern

end namespace

#endif ' __rp3d_proxyshape__
