#ifndef __rp3d_rigidbody__
#define __rp3d_rigidbody__


namespace rp3d

extern "C"

' ###################################
' # CollisionBody extends Body      #
' # RigidBody extends CollisionBody #
' ###################################
' This class represents a rigid body of the physics engine.
' A rigid body is a non-deformable body that has a constant mass.
' This class inherits from the CollisionBody class.

' methods

' Set the type of the body
' The type of the body can either BT_STATIC, BT_KINEMATIC or BT_DYNAMIC as described bellow:

' BT_STATIC:    A static body has infinite mass, zero velocity but the position can be changed manually.
'               A static body does not collide with other static or kinematic bodies.

' BT_KINEMATIC: A kinematic body has infinite mass, the velocity can be changed manually
'               and its position is computed by the physics engine.
'               A kinematic body does not collide with other static or kinematic bodies.

' BT_DYNAMIC:   A dynamic body has non-zero mass, non-zero velocity determined by forces
'               and its position is determined by the physics engine.
'               A dynamic body can collide with other dynamic, static or kinematic bodies.
declare sub      RigidBodySetType(byval rb as RigidBody ptr,byval bt as eBodyType)
declare function RigidBodyGetType(byval rb as const RigidBody ptr) as eBodyType

' Return the ID of the body
declare function RigidBodyGetId alias "BodyGetId" (byval rb as RigidBody ptr) as uinteger ' 32/64-bit

' Return whether or not the body is allowed to sleep
declare function RigidBodyIsAllowedToSleep alias "BodyIsAllowedToSleep" (byval rb as RigidBody ptr) as boolean
' Set whether or not the body is allowed to go to sleep
declare sub RigidBodySetIsAllowedToSleep alias "BodySetIsAllowedToSleep" (byval rb as RigidBody ptr, byval isAllowedToSleep as boolean)

' Return true if the body is active
declare function RigidBodyIsActive alias "BodyIsActive" (byval rb as RigidBody ptr) as boolean
' Set whether or not the body is active
declare sub RigidBodySetIsActive alias "BodySetIsActive" (byval rb as RigidBody ptr, byval isActive as boolean)

' Return the user data attached to this body
declare function RigidBodyGetUserData alias "BodyGetUserData" (byval rb as RigidBody ptr) as any ptr
' Attach user data to this body
declare sub RigidBodySetUserData alias "BodySetUserData" (byval rb as RigidBody ptr, byval userData as any ptr)

' Return true if the gravity needs to be applied to this rigid body
declare function RigidBodyIsGravityEnabled(byval rb as RigidBody ptr) as boolean
' Set the variable to know if the gravity is applied to this rigid body
declare sub      RigidBodyEnableGravity(byval rb as RigidBody ptr, byval isEnabled as boolean)

' Set the variable to know whether or not the body is sleeping
declare sub      RigidBodySetIsSleeping(byval rb as RigidBody ptr, byval isSleeping as boolean)

' Return a reference to the material properties of the rigid body
declare function RigidBodyGetMaterial(byval rb as RigidBody ptr) byref as Material
' Set a new material for this rigid body
declare sub      RigidBodySetMaterial(byval rb as RigidBody ptr, byref material as const Material)

' Return the mass of the body
declare function RigidBodyGetMass(byval rb as RigidBody ptr) as decimal
' Set the mass of the rigid body
declare sub      RigidBodySetMass(byval rb as RigidBody ptr, byval mass as decimal)

' Set the current position and orientation
declare sub      RigidBodySetTransform(byval rb as RigidBody ptr, _
                                       byref t as const Transform)
declare sub      RigidBodySetTransformPosRotXYZ(byval rb as RigidBody ptr, _
                                                byval px as decimal, byval py as decimal, byval pz as decimal, _
                                                byval rx as decimal, byval ry as decimal, byval rz as decimal)
declare sub      RigidBodySetTransformPosRot(byval rb as RigidBody ptr, _
                                             byref position as const Vector3, _
                                             byref rotation as const Vector3)
' Get the current position and orientation
declare function RigidBodyGetTransform alias "CollisionBodyGetTransform" (byval rb as RigidBody ptr) byref as const Transform

' Return true if a point is inside the collision body
declare function RigidBodyTestPointInside alias "CollisionBodyTestPointInside" (byval rb as RigidBody ptr, _
                                                                                byref worldPoint as const Vector3) as boolean
' Raycast method with feedback information
declare function RigidBodyRaycast alias "CollisionBodyRaycast" (byval rb as RigidBody ptr, _
                                                                byref ray as const Ray, _
                                                                byref raycastInfo as RaycastInfo) as boolean
' Test if the collision body overlaps with a given AABB
declare function RigidBodyTestAABBOverlap alias "CollisionBodyTestAABBOverlap" (byval rb as RigidBody ptr, _
                                                                                byref worldAABB as const AABB) as boolean

' Compute and return the AABB of the body by merging all proxy shapes AABBs
declare function RigidBodyGetAABB alias "CollisionBodyGetAABB" (byval rb as RigidBody ptr) as AABB

' Return the linked list of proxy shapes of that body
declare function RigidBodyGetProxyShapesList alias "CollisionBodyGetProxyShapesList" (byval rb as RigidBody ptr) as ProxyShape ptr

' Return the "const" linked list of proxy shapes of that body
declare function RigidBodyGetProxyShapesListC alias "CollisionBodyGetProxyShapesListC" (byval rb as RigidBody ptr) as const ProxyShape ptr

' Return the world-space coordinates of a point given the local-space coordinates of the body
declare function RigidBodyGetWorldPoint alias "CollisionBodyGetWorldPoint" (byval rb as RigidBody ptr, byref localPoint as const Vector3) as Vector3

' Return the body local-space coordinates of a point given in the world-space coordinates
declare function RigidBodyGetLocalPoint alias "CollisionBodyGetLocalPoint" (byval rb as RigidBody ptr, byref worldPoint as const Vector3) as Vector3

' Return the world-space vector of a vector given in local-space coordinates of the body 
declare function RigidBodyGetWorldVector alias "CollisionBodyGetWorldVector" (byval rb as RigidBody ptr, byref localVector as const Vector3) as Vector3

' Return the body local-space coordinates of a vector given in the world-space coordinates
declare function RigidBodyGetLocalVector alias "CollisionBodyGetLocalVector" (byval rb as RigidBody ptr, byref worldVector as const Vector3) as Vector3

' Return the linear velocity
declare function RigidBodyGetLinearVelocity(byval rb as RigidBody ptr) as Vector3
declare sub      RigidBodyGetLinearVelocityXYZ(byval rb as RigidBody ptr, byref x as decimal, byref y as decimal, byref z as decimal)
'  Set the linear velocity of the body.
declare sub      RigidBodySetLinearVelocity(byval rb as RigidBody ptr, byref v as const Vector3)
declare sub      RigidBodySetLinearVelocityXYZ(byval rb as RigidBody ptr, byval x as decimal, byval y as decimal, byval z as decimal)

' Return the angular velocity
declare function RigidBodyGetAngularVelocity(byval rb as RigidBody ptr) as Vector3
declare sub      RigidBodyGetAngularVelocityXYZ(byval rb as RigidBody ptr, byref x as decimal, byref y as decimal, byref z as decimal)

' Set the angular velocity.
declare sub      RigidBodySetAngularVelocity(byval rb as RigidBody ptr, byref angularVelocity as const Vector3)
declare sub      RigidBodySetAngularVelocityXYZ(byval rb as RigidBody ptr, byval x as decimal, byval y as decimal, byval z as decimal)

' Set the local inertia tensor of the body (in body coordinates)
declare sub      RigidBodySetInertiaTensorLocal(byval rb as RigidBody ptr, byref inertiaTensorLocal as const Matrix3x3)

' Set the inverse local inertia tensor of the body (in body coordinates)
declare sub      RigidBodySetInverseInertiaTensorLocal(byval rb as RigidBody ptr, byref inverseInertiaTensorLocal as const Matrix3x3)
' Get the inverse local inertia tensor of the body (in body coordinates)
declare function RigidBodyGetInverseInertiaTensorLocal(byval rb as RigidBody ptr) byref as const Matrix3x3

' Return the inverse of the inertia tensor in world coordinates.
declare function RigidBodyGetInertiaTensorInverseWorld(byval rb as RigidBody ptr) as Matrix3x3
' Set the local center of mass of the body (in local-space coordinates)
declare sub      RigidBodySetCenterOfMassLocal(byval rb as RigidBody ptr, byref centerOfMassLocal as const Vector3)
declare sub      RigidBodySetCenterOfMassLocalXYZ(byval rb as RigidBody ptr, byval x as decimal, byval y as decimal, byval z as decimal)


' Return the linear velocity damping factor
declare function RigidBodyGetLinearDamping(byval rb as RigidBody ptr) as decimal
' Set the linear damping factor
declare sub      RigidBodySetLinearDamping(byval rb as RigidBody ptr, byval linearDamping as decimal)

' Return the angular velocity damping factor
declare function RigidBodyGetAngularDamping(byval rb as RigidBody ptr) as decimal
' Set the angular damping factor
declare sub      RigidBodySetAngularDamping(byval rb as RigidBody ptr, byval angularDamping as decimal)

' Apply an external force to the body at its center of mass.
declare sub      RigidBodyApplyForceToCenterOfMass(byval rb as RigidBody ptr, byref force as const Vector3)
declare sub      RigidBodyApplyForceToCenterOfMassXYZ(byval rb as RigidBody ptr, byval x as decimal, byval y as decimal, byval z as decimal)

' Apply an external force to the body at a given point (in world-space coordinates).
declare sub      RigidBodyApplyForce(byval rb as RigidBody ptr, byref force as const Vector3, byref worldPoint as const Vector3)
declare sub      RigidBodyApplyForceXYZ(byval rb as RigidBody ptr, _
                                        byval fx as decimal, byval fy as decimal, byval fz as decimal, _
                                        byval px as decimal, byval py as decimal, byval pz as decimal)
' Apply an external torque to the body.
declare sub      RigidBodyApplyTorque(byval rb as RigidBody ptr, byref torque as const Vector3)
declare sub      RigidBodyApplyTorqueXYZ(byval rb as RigidBody ptr, byval x as decimal, byval y as decimal, byval z as decimal)

' Return the first element of the linked list of joints involving this body
declare function RigidBodyGetJointsList(byval rb as RigidBody ptr) as JointListElement ptr
' Return the first element of the linked list of joints involving this body
declare function RigidBodyGetJointsListC(byval rb as RigidBody ptr) as const JointListElement ptr



' Add a collision shape to the body.
' Note that you can share a collision shape between several bodies using the same 
' collision shape instance to when you add the shape to the different bodies.
' Do not forget to delete the collision shape you have created at the end of your program.

' This method will return a pointer to a new proxy shape.
' A proxy shape is an object that links a collision shape and a given body.
' You can use the returned proxy shape to get and set information 
' about the corresponding collision shape for that body.

' ProxyShape constructors
' Add a collision shape to the body.
declare function RigidBodyAddCollisionShapePosRotXYZ(byval rb as RigidBody ptr, _
                                                     byval cs as CollisionShape ptr, _
                                                     byval px as decimal, byval py as decimal, byval pz as decimal, _
                                                     byval rx as decimal, byval ry as decimal, byval rz as decimal, _
                                                     byval mass as decimal) as ProxyShape ptr

declare function RigidBodyAddCollisionShapePosRot(byval rb as RigidBody ptr, _
                                                  byval cs as CollisionShape ptr, _
                                                  byref position as const Vector3, _
                                                  byref rotation as const Vector3, _
                                                  byval mass as decimal) as ProxyShape ptr

declare function RigidBodyAddCollisionShape(byval rb as RigidBody ptr, _
                                            byval cs as CollisionShape ptr, _
                                            byref t  as const Transform, _
                                            byval mass as decimal) as ProxyShape ptr

declare function RigidBodyAddCollisionShapeAtCenter(byval rb as RigidBody ptr, _
                                                    byval cs as CollisionShape ptr, _
                                                    byval mass as decimal) as ProxyShape ptr

declare sub      RigidBodyRemoveCollisionShape(byval rb as RigidBody ptr, _
                                               byval proxyShape as const ProxyShape ptr)

end extern

end namespace

#endif ' __rp3d_rigidbody__
